<?php

namespace LWM\PTI;

if ( ! defined( 'WPINC' ) ) {
	die;
}

class Canvas {

    public int $post_id;
    public bool $save_image = false;
    public bool $set_alt_tag = false;

    public function __construct( $post_id, $save_image, $set_alt_tag ) {
        $this->post_id = $post_id;        
        $this->save_image = $save_image;
        $this->set_alt_tag = $set_alt_tag;
    }

    public function init() {
        $image = $this->generateImage();

        if( !empty( $this->save_image ) ) {
            $this->save( $image['image'], $image['title'] );
        }

        return $image['image'];
    }

    /**
     * save
     * 
     */
    public function save( $image, $title ) {
        $old_attachment_id = get_post_thumbnail_id( $this->post_id );

        // delete old attachment
        if( !empty( $old_attachment_id ) ) {
            $is_post_to_image = get_post_meta( $old_attachment_id, '_is_post_to_image', true );
            if( $is_post_to_image ) {
                wp_delete_attachment( $old_attachment_id, true );
            }
        }

        $upload_dir  = wp_upload_dir();
        $upload_path = str_replace( '/', DIRECTORY_SEPARATOR, $upload_dir['path'] ) . DIRECTORY_SEPARATOR;
        $img = str_replace( 'data:image/jpeg;base64,', '', $image );
        $img = str_replace( ' ', '+', $img );
        $decoded = base64_decode( $img );
        // $filename = get_post_field( 'post_name', $this->post_id ) . '.jpeg';
        $filename = sanitize_title( $title ) . '.jpeg';
        $file_type = 'image/jpeg';

        $upload_file = file_put_contents( $upload_path . $filename, $decoded );

        $attachment = [
            'post_mime_type' => $file_type,
            'post_title' => basename( $filename ),
            'post_content' => '',
            'post_status' => 'inherit',
            'guid' => $upload_dir['url'] . '/' . basename( $filename )
        ];

        $attach_id = wp_insert_attachment( $attachment, $upload_path . $filename );
        $attachment_meta = wp_generate_attachment_metadata( $attach_id, $upload_path . $filename );
        wp_update_attachment_metadata( $attach_id, $attachment_meta );
        set_post_thumbnail( $this->post_id, $attach_id );

        $old_attachment_id = $attach_id;

        // set image alt
        if( !empty( $this->set_alt_tag ) && !empty( $old_attachment_id ) ) {  
            update_post_meta( 
                $old_attachment_id, 
                '_wp_attachment_image_alt', 
                sanitize_text_field( TTPI_CANVAS_ALT_PREFIX . sanitize_text_field( (string)trim($title) ) . TTPI_CANVAS_ALT_SUFFIX )
            );
        }

        update_field('field_61f0178b5f15e', '', $this->post_id);
        update_field('field_633556d81b10e', '', $this->post_id);

        // post meta for images
        update_post_meta( $attach_id, '_is_post_to_image', '1' );  
        // post meta for posts
        update_post_meta( $this->post_id, '_is_post_to_image_init', '1' ); 
    }

    /**
     * generate image
     * 
     */
    private function generateImage() {

        $img = imagecreatetruecolor( TTPI_CANVAS_WIDTH, TTPI_CANVAS_HEIGHT );

        // background
        $this->gradient( 
            $img, 
            0, 
            0, 
            TTPI_CANVAS_WIDTH, 
            TTPI_CANVAS_HEIGHT, 
            str_replace( '#', '', TTPI_CANVAS_GRADIENT_START ), 
            str_replace( '#', '', TTPI_CANVAS_GRADIENT_STOP )
        );

        // title
        $text_color = imagecolorallocate( $img, 255, 255, 255 );

        if( TTPI_CANVAS_FONT_COLOR ) {
            $rgb = $this->hexToRgb( TTPI_CANVAS_FONT_COLOR );
            $text_color = imagecolorallocate( $img, $rgb['r'], $rgb['g'], $rgb['b'] );
        }
        
        $font_path = __DIR__ . '/../../fonts/open-sans/OpenSans-Bold.ttf';
        $font_path_small = __DIR__ . '/../../fonts/open-sans/OpenSans-Regular.ttf';

        if( TTPI_CANVAS_FONT_FAMILY === 'inter' ) {
            $font_path = __DIR__ . '/../../fonts/inter/Inter-Bold.ttf';
            $font_path_small = __DIR__ . '/../../fonts/inter/Inter-Medium.ttf';
        }

        $tmp_title = $this->title()['title'];
        $wrap_y = 0;

        // layout: custom
        if( TTPI_LAYOUT == 2 && !get_field( 'base64_title', $this->post_id ) ) {
            imagettftext( 
                $img, (int)TTPI_CANVAS_FONT_SIZE-8, 0, TTPI_CANVAS_WRAP_X, TTPI_CANVAS_WRAP_Y, 
                $text_color, $font_path_small, $this->title()['prefix'] 
            );
            $wrap_y = 50;
            $tmp_title = $this->title()['title_org'];
        }

        $tmp_title =  sanitize_text_field( $this->removeEmoji( $tmp_title ) );
        $title = wordwrap(
            $tmp_title,
            TTPI_CANVAS_FONT_BREAK,
            "\n",
            true // cut long words
        );

        imagettftext( 
            $img, 
            (float)str_replace( 'px', '', TTPI_CANVAS_FONT_SIZE ), 
            0, TTPI_CANVAS_WRAP_X, TTPI_CANVAS_WRAP_Y + $wrap_y, $text_color, $font_path, $title 
        );

        ob_start();

        imagejpeg( $img, null, 100 );

        $result = ob_get_contents();
	    ob_end_clean();

        return [
            'image' => 'data:image/jpeg;base64,' . base64_encode( $result ),
            'title' => $title
        ];
    }

    /**
     * remove emojis and other special chars
     * 
     */
    private function removeEmoji( $text ) {
        $text = iconv('UTF-8', 'ISO-8859-15//IGNORE', $text);
        $text = preg_replace('/\s+/', ' ', $text);
    
        return iconv('ISO-8859-15', 'UTF-8', $text);
    }

    /**
     * build title
     * 
     */
    private function title() {
        $categories = $this->setCategory( get_the_category( $this->post_id ) );
        $prefix = '';
        $suffix = '';

        // post title or custom title (acf)
        $post_title = get_the_title( $this->post_id );
        $custom_post_title = get_field( 'base64_title', $this->post_id );
        
        if( !empty( $custom_post_title ) ) {
            $post_title = $custom_post_title;
        } else {
            $prefix .= str_replace( 'category', $categories, TTPI_CANVAS_PREFIX ) . TTPI_CANVAS_PREFIX_DELIMITER;
            $suffix .= TTPI_CANVAS_SUFFIX_DELIMITER . str_replace( 'category', $categories, TTPI_CANVAS_SUFFIX );
        }

        $title = $prefix . html_entity_decode( $post_title ) . $suffix;

        return [
            'title'     => trim( \strip_shortcodes($title) ),
            'title_org' => html_entity_decode( trim( \strip_shortcodes($post_title) ) ),
            'prefix'    => $prefix,
            'suffix'    => $suffix
        ];
    }

    /**
     * hex to rgb
     * 
     */
    private function hexToRgb( $hex ) {
        $hex = str_replace( '#', '', $hex );
        $hex = trim( $hex );

        $rgb = [
            'r' => 255,
            'g' => 255,
            'b' => 255
        ];

        if( strlen( $hex ) === 6 ) {
            $rgb['r'] = hexdec( substr($hex, 0, 2) );
            $rgb['g'] = hexdec( substr($hex, 2, 2) );
            $rgb['b'] = hexdec( substr($hex, 4, 2) );
        }

        return $rgb;
    }

    /**
     * categories
     * 
     */
    private function setCategory( $categories ) {
        $content = '';
    
        if( !empty( $categories ) ) {
            foreach( $categories as $category ) {
                $content .= $category->name . ', ';
            }
        }
    
        return trim( $content, ', ' );
    }

    /**
     * gradient
     * 
     */
    private function gradient( $img, $x, $y, $x1, $y1, $start, $end ) {
        if($x > $x1 || $y > $y1) {
            return false;
        }
        $s = array(
            hexdec(substr($start,0,2)),
            hexdec(substr($start,2,2)),
            hexdec(substr($start,4,2))
        );
        $e = array(
            hexdec(substr($end,0,2)),
            hexdec(substr($end,2,2)),
            hexdec(substr($end,4,2))
        );

        // $steps = $y1 - $y; // vertical
        $steps = $x1 - $x; // horizontal

        for($i = 0; $i < $steps; $i++) {
            $r = $s[0] - ((($s[0]-$e[0])/$steps)*$i);
            $g = $s[1] - ((($s[1]-$e[1])/$steps)*$i);
            $b = $s[2] - ((($s[2]-$e[2])/$steps)*$i);
            $color = imagecolorallocate( $img, (int) $r, (int) $g, (int) $b );
            imagefilledrectangle( $img, $x+$i, $y, $x1+$i+1, $y1, $color ); // horizontal
        }
        
        return true;
    }

}