<?php

namespace LWM\PTI;
use LWM\PTI\Canvas;

if ( ! defined( 'WPINC' ) ) {
	die;
}

class Initial {

    public function init() {     
        add_action( 'wp_ajax_load', [ $this, 'load' ] );
        // add_action( 'wp_ajax_nopriv_load', [ $this, 'load' ] );
    }

    public function load() {

        $data = $_GET;

        if ( ! wp_verify_nonce( $data['_ajax_nonce'], 'ttpi-nonce' ) ) {
            $return = [
                'success' => false
            ];
        } else {
            if( $_GET['type'] === 'set' ) {
                $return = [
                    'success' => true, 
                    'return' => $this->set()
                ];
            } elseif( $_GET['type'] === 'reset' ) {
                $return = [
                    'success' => true, 
                    'return' => $this->reset()
                ];
            } else {
                $return = [
                    'success' => false, 
                    'reason' => 'wrong type'
                ];
            }
            
        }

        echo json_encode( $return );

        wp_die();
    }

    /**
     * reset
     * 
     */
    private function reset() {
        $return = [];

        $post = new \WP_Query([
            'post_type' => 'any',
            'posts_per_page' => 1,
            'meta_query' => [
                [
                    'key' => '_is_post_to_image_init',
                    'compare' => 'EXISTS'
                ]
            ]
        ]);

        $foundPosts = ( $post->found_posts - 1 );

        $allPosts = $this->getAllPosts( false );

        // var_dump($allPosts, $foundPosts);
        // wp_die();

        if ( $post->have_posts() ) {
            while ( $post->have_posts() ) {
                $post->the_post();

                if( get_post_meta( get_post_thumbnail_id(), '_is_post_to_image', true ) ) {
                    wp_delete_attachment( get_post_thumbnail_id(), true );
                }

                delete_post_meta( get_the_ID(), '_is_post_to_image_init' );
            }

            $return = [
                'posts_all'     => $allPosts,
                'posts_left'    => $foundPosts,
                'progress'      => ( 100 * ( $allPosts-$foundPosts ) ) / $allPosts
            ];
        }

        return $return;
    }

    /**
     * set
     * 
     */
    private function set() {
        $return = [];

        $post = new \WP_Query([
            'post_type' => TTPI_POST_TYPES,
            'posts_per_page' => 1,
            'meta_query' => [
                [
                    'key' => '_is_post_to_image_init',
                    'compare' => 'NOT EXISTS'
                ]
            ]
        ]);

        $allPosts = $this->getAllPosts();

        if ( $post->have_posts() ) {
            while ( $post->have_posts() ) {
                $post->the_post();
                $canvas = new Canvas( 
                    get_the_ID(),
                    true,
                    true
                );
                $canvas = $canvas->init();
            }

            $foundPosts = ( $post->found_posts - 1 );

            $return = [
                'posts_all'     => $allPosts,
                'posts_left'    => $foundPosts,
                'progress'      => ( 100 * ( $allPosts-$foundPosts ) ) / $allPosts
            ];
        }

        return $return;
    }

    /**
     * get all posts
     * 
     */
    private function getAllPosts( $with_post_types = true ) {
        $args = [
            'post_type' => 'any',
            'posts_per_page' => 1,
        ];

        if( !empty( $with_post_types ) ) {
            $args['post_type'] = TTPI_POST_TYPES;
        }

        $post = new \WP_Query($args);

        return $post->found_posts;
    }

}