<?php

namespace LWM\PTI;

if ( ! defined( 'WPINC' ) ) {
	die;
}

class Menu {

    public function init() {
        add_action('admin_menu', [ $this, 'getMenu' ] );
        add_action('admin_init', [ $this, 'setSettings' ] );
    }

    public function getMenu() {

        add_menu_page(
            'Post to image', 
            'Post to image', 
            'manage_options', 
            'title_to_postimage', 
            [ $this, 'options' ]
        );

        add_submenu_page(
			'title_to_postimage',
			'Initial loading',
			'Initial loading',
			'manage_options',
			'initLoading',
			[ $this, 'initialLoading' ] );

    }

    /**
     * Main Page
     * include partial
     * 
     */
    public function options() {
        require_once __DIR__ . '../../../partials/admin/main.php';
    }

    /**
     * Initial Loading Page
     * include partial
     */
    public function initialLoading() {
        require_once __DIR__ . '../../../partials/admin/initial.php';
    }

    /**
     * init settings
     * 
     */
    public function setSettings() {
        register_setting( 
            'ttpi', 
            'ttpi_settings',
            [ $this, 'ttpi_sanitize' ]
        );

        add_settings_section(
            'ttpi_section', 
            '', 
            '__return_false', 
            'ttpi'
        );

        add_settings_field( 
            'ttpi_post_types', 
            __( 'Post Types:', 'title_to_postimage' ), 
            [ $this, 'ttpi_post_types' ],
            'ttpi', 'ttpi_section' 
        );

        add_settings_field( 
            'ttpi_layout', 
            __( 'Layout:', 'title_to_postimage' ), 
            [ $this, 'ttpi_layout' ],
            'ttpi', 'ttpi_section' 
        );

        add_settings_field( 
            'ttpi_canvas_alt_prefix', 
            __( 'Alt-Tag Prefix:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_alt_prefix' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_alt_suffix', 
            __( 'Alt-Tag Suffix:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_alt_suffix' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_prefix', 
            __( 'Title Prefix:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_prefix' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_prefix_delimiter', 
            __( 'Title Prefix Delimiter:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_prefix_delimiter' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_suffix', 
            __( 'Title Suffix:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_suffix' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_suffix_delimiter', 
            __( 'Title Suffix Delimiter:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_suffix_delimiter' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_width', 
            __( 'Canvas Width:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_width' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_height', 
            __( 'Canvas Height:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_height' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_gradient_start', 
            __( 'Canvas Gradient Start:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_gradient_start' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_gradient_stop', 
            __( 'Canvas Gradient Stop:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_gradient_stop' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_font_family', 
            __( 'Canvas Font Family:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_font_family' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_font_size', 
            __( 'Canvas Font Size:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_font_size' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_font_weight', 
            __( 'Canvas Font Weight:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_font_weight' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_font_color', 
            __( 'Canvas Font Color:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_font_color' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_wrap_x', 
            __( 'Canvas WRAP X:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_wrap_x' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_wrap_y', 
            __( 'Canvas WRAP Y:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_wrap_y' ],
            'ttpi', 'ttpi_section' 
        );
        add_settings_field( 
            'ttpi_canvas_font_break', 
            __( 'Canvas Font Break:', 'title_to_postimage' ), 
            [ $this, 'ttpi_canvas_font_break' ],
            'ttpi', 'ttpi_section' 
        );
        
    }

    public function ttpi_sanitize( $input ) {
        $input["ttpi_post_types"] = maybe_serialize($input["ttpi_post_types"]);
        // var_dump( $input );

        // wp_die();
        return $input;
    }

    /**
     * field: post types
     */
    public function ttpi_post_types() {
        $options = get_option( 'ttpi_settings' );

        $post_types = get_post_types( 
			[
				'public' => true,
       			'_builtin' => false,
			], 
			'objects', 
			'and' 
		);
        ?>
        
        <?php if( is_array( TTPI_POST_TYPES ) ): ?>
		<select style="width: 100%;" name="ttpi_settings[ttpi_post_types][]" multiple>
			<option value="post" <?php echo ( in_array('post', TTPI_POST_TYPES) ? 'selected' : '' ); ?>>Posts</option>
			<option value="page" <?php echo ( in_array('page', TTPI_POST_TYPES) ? 'selected' : '' ); ?>>Pages</option>

			<?php foreach( $post_types as $type => $t ): ?>
				<option value="<?php echo $t->name; ?>" <?php echo ( in_array($t->name, TTPI_POST_TYPES) ? 'selected' : '' ); ?>><?php echo $t->label; ?></option>
			<?php endforeach; ?>
		</select>
		<?php endif; ?>

        <?php
    }

    /**
     * field: layout
     */
    public function ttpi_layout() { 
        $options = [
            '1' => __( 'Standard', 'title_to_postimage' ),
            '2' => __( 'Prefix oberhalb, ignoriere Suffix', 'title_to_postimage' )
        ];
        ?>
        <select name="ttpi_settings[ttpi_layout]" autocomplete="off">
            <?php
            foreach( $options as $option => $o ) {
                echo '<option value="' . $option . '" ' . ( !empty( TTPI_LAYOUT ) && $option == TTPI_LAYOUT ? 'selected' : '' ) . '>' . $o . '</option>';
            }
            ?>
        </select>
        <?php
    }

    /**
     * field: canvas prefix
     */
    public function ttpi_canvas_prefix() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_prefix]' 
            value='<?php echo TTPI_CANVAS_PREFIX; ?>'>
        <p>Setze "category" für die Kategorie</p>
        <?php
    }

    /**
     * field: canvas prefix delimiter
     */
    public function ttpi_canvas_prefix_delimiter() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_prefix_delimiter]' 
            value='<?php echo TTPI_CANVAS_PREFIX_DELIMITER; ?>'>
        <p>Leerzeichen setzen, wenn nötig</p>
        <?php
    }

    /**
     * field: canvas suffix
     */
    public function ttpi_canvas_suffix() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_suffix]' 
            value='<?php echo TTPI_CANVAS_SUFFIX; ?>'>
        <p>Setze "category" für die Kategorie</p>
        <?php
    }

    /**
     * field: canvas suffix delimiter
     */
    public function ttpi_canvas_suffix_delimiter() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_suffix_delimiter]' 
            value='<?php echo TTPI_CANVAS_SUFFIX_DELIMITER; ?>'>
        <p>Leerzeichen setzen, wenn nötig</p>
        <?php
    }

    /**
     * field: canvas alt prefix
     */
    public function ttpi_canvas_alt_prefix() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_alt_prefix]' 
            value='<?php echo TTPI_CANVAS_ALT_PREFIX; ?>'>
        <p>Leerzeichen & Delimiter setzen, wenn nötig</p>
        <?php
    }

    /**
     * field: canvas alt suffix
     */
    public function ttpi_canvas_alt_suffix() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_alt_suffix]' 
            value='<?php echo TTPI_CANVAS_ALT_SUFFIX; ?>'>
        <p>Leerzeichen & Delimiter setzen, wenn nötig</p>
        <?php
    }

    /**
     * field: canvas width
     */
    public function ttpi_canvas_width() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='number' min="1" step="1" 
            name='ttpi_settings[ttpi_canvas_width]' 
            value='<?php echo TTPI_CANVAS_WIDTH; ?>'>
        <?php
    }

    /**
     * field: canvas height
     */
    public function ttpi_canvas_height() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='number' min="1" step="1" 
            name='ttpi_settings[ttpi_canvas_height]' 
            value='<?php echo TTPI_CANVAS_HEIGHT; ?>'>
        <?php
    }

    /**
     * field: canvas gradient start
     */
    public function ttpi_canvas_gradient_start() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_gradient_start]' 
            value='<?php echo TTPI_CANVAS_GRADIENT_START; ?>'>
        <?php
    }

    /**
     * field: canvas gradient end
     */
    public function ttpi_canvas_gradient_stop() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_gradient_stop]' 
            value='<?php echo TTPI_CANVAS_GRADIENT_STOP; ?>'>
        <?php
    }

    /**
     * field: canvas font family
     */
    public function ttpi_canvas_font_family() { 
        $fonts = [
            'opensans'  => 'Open Sans',
            'inter'     => 'Inter'
        ];
        ?>
        <select name="ttpi_settings[ttpi_canvas_font_family]" autocomplete="off">
            <?php
            foreach( $fonts as $font => $f ) {
                echo '<option value="' . $font . '" ' . ( !empty( TTPI_CANVAS_FONT_FAMILY ) && TTPI_CANVAS_FONT_FAMILY == $font ? 'selected' : '' ) . '>' . $f . '</option>';
            }
            ?>
        </select>
        <?php
    }

    /**
     * field: canvas font size
     */
    public function ttpi_canvas_font_size() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='number' 
            min="1"
            step="1"
            name='ttpi_settings[ttpi_canvas_font_size]' 
            value='<?php echo TTPI_CANVAS_FONT_SIZE; ?>'>
        <?php
    }

    /**
     * field: canvas font weight
     */
    public function ttpi_canvas_font_weight() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_font_weight]' 
            value='<?php echo TTPI_CANVAS_FONT_WEIGHT; ?>'>
        <?php
    }

    /**
     * field: canvas font color
     */
    public function ttpi_canvas_font_color() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='text' 
            name='ttpi_settings[ttpi_canvas_font_color]' 
            value='<?php echo TTPI_CANVAS_FONT_COLOR; ?>'>
        <?php
    }

    /**
     * field: canvas wrap x
     */
    public function ttpi_canvas_wrap_x() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='number' min="0" step="1" 
            name='ttpi_settings[ttpi_canvas_wrap_x]' 
            value='<?php echo TTPI_CANVAS_WRAP_X; ?>'>
        <p>Startpunkt des Titels</p>
        <?php
    }

    /**
     * field: canvas wrap y
     */
    public function ttpi_canvas_wrap_y() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='number' min="0" step="1" 
            name='ttpi_settings[ttpi_canvas_wrap_y]' 
            value='<?php echo TTPI_CANVAS_WRAP_Y; ?>'>
        <p>Startpunkt des Titels</p>
        <?php
    }

    /**
     * field: canvas wrap max width
     */
    public function ttpi_canvas_font_break() { 
        $options = get_option( 'ttpi_settings' );
        ?>
        <input type='number' min="0" step="1" 
            name='ttpi_settings[ttpi_canvas_font_break]' 
            value='<?php echo TTPI_CANVAS_FONT_BREAK; ?>'>
        <p>Zeilenumbruch</p>
        <?php
    }

}