const { subscribe, select, dispatch } = wp.data;
const { isSavingPost } = select( 'core/editor' );

import './scss/main.scss';

const wrapText = ( context, text, x, y, maxWidth, lineHeight ) => {
    var words = text.split(' ');
    var line = '';

    for(var n = 0; n < words.length; n++) {
      var testLine = line + words[n] + ' ';
      var metrics = context.measureText(testLine);
      var testWidth = metrics.width;
      if (testWidth > maxWidth && n > 0) {
        context.fillText(line, x, y);
        line = words[n] + ' ';
        y += lineHeight;
      }
      else {
        line = testLine;
      }
    }
    context.fillText(line, x, y);
}

const GenerateCanvas = ( title, base64 ) => {
    // base64.value = title;
    let c = document.createElement('canvas');
    c.width = ttpi.canvas.width;
    c.height = ttpi.canvas.height;
    let ctx = c.getContext('2d');

    // Background
    const gradient = ctx.createLinearGradient(0 , 0, c.width, 0);
    gradient.addColorStop(0, ttpi.canvas.gradient.start);
    gradient.addColorStop(1, ttpi.canvas.gradient.stop);
    ctx.fillStyle = gradient;
    ctx.fillRect(0, 0, c.width, c.height);

    // Text
    // ctx.font = 'bold 45px Open Sans';
    ctx.font = `${ttpi.canvas.font.weight} ${ttpi.canvas.font.size}px ${ttpi.canvas.font.family}`;
    ctx.fillStyle = ttpi.canvas.font.color;
    wrapText(ctx, title, ttpi.canvas.wrap.x, ttpi.canvas.wrap.y, ttpi.canvas.wrap.maxwidth, ttpi.canvas.wrap.lineheight);

    const stream = c.toDataURL("image/png");
    base64.value = stream;

    return stream; 
}

let is_saving = false;

const logTitle = () => {
    let title = select("core/editor").getEditedPostAttribute( 'title' );
    const post_type = select('core/editor').getCurrentPostType();
    const Base64Container = document.querySelector('.acf-field-61efe4beca622');
    const Base64Input = document.getElementById('acf-field_61efe4beca622');
    const CanvasContainer = document.getElementById("CanvasContainer");
    const TitleContainer = document.getElementById('acf-field_61f01fd700db1');

    const selectedCatIds = select("core/editor").getEditedPostAttribute("categories");
    const allCats = select('core').getEntityRecords('taxonomy', 'category');    

    if( select("core/editor").isSavingPost() ) {
        is_saving = true
    }

    if( is_saving && select("core/editor").isSavingPost() === false ) {
        is_saving = false
        setTimeout( () => {
            document.getElementById('acf-field_61f0178b5f15e-yes').checked = false
            // document.getElementById('acf-field_633556d81b10e-yes').checked = false
        }, 1000)
    }

    if( post_type === 'post' ) {
        let cat = '';
        if( selectedCatIds && allCats ) {
            allCats.forEach( item => {
                if( selectedCatIds.includes(item.id) ) {
                    cat += item.name + ', ';
                }
            })
            cat = cat.slice( 0, -2 );
        }

        let prefix = ''
        let suffix = ''

        if( ttpi.canvas.prefix === 'category' && cat ) {
            prefix = `${ cat }${ ttpi.canvas.prefix_delimiter }`
        }
        if( ttpi.canvas.prefix && ttpi.canvas.prefix !== 'category' ) {
            prefix = `${ ttpi.canvas.prefix }${ ttpi.canvas.prefix_delimiter }`
        }
        if( ttpi.canvas.suffix === 'category' && cat ) {
            suffix = `${ ttpi.canvas.suffix_delimiter }${ cat }`
        }
        if( ttpi.canvas.suffix && ttpi.canvas.suffix !== 'category' ) {
            suffix = `${ ttpi.canvas.suffix_delimiter }${ ttpi.canvas.suffix }`
        }

        title = prefix + title + suffix
    }


    if( CanvasContainer && Base64Container ) {
        Base64Container.removeChild( CanvasContainer );
    }

    if( Base64Input && title ) {
        let streamOutput = GenerateCanvas( title, Base64Input );

        if( TitleContainer ) {
            const TitleContent = title;
            TitleContainer.value = TitleContent; 
        }

        // preview
        let preview = document.createElement("div");
        preview.setAttribute("id", "CanvasContainer");
        preview.innerHTML = '<img src="' + streamOutput + '" />';
        Base64Container.appendChild( preview );
    }
}

subscribe( logTitle )