<?php
/**
 * Plugin Name: Title to featured Image
 * Description: Generiert aus dem Titel ein Bild und speichert es als Beitragsbild.
 * Version: 2.3.1
 * Author: Sven Hoffmann | Lightweb Media GmbH
 * Author URI: https://lightweb-media.de
 * Text Domain: title_to_postimage
 * Requires at least: 5.4.0
 * Requires PHP: 7.4.0
 * Tested up to: 6.6.1
 * Update URI: https://plugins.lightweb-media.de/api?action=get_metadata&slug=title_to_postimage_plugin
 */

 defined( 'ABSPATH' ) || exit;

 use YahnisElsts\PluginUpdateChecker\v5\PucFactory;

 require_once __DIR__ . '/define.php';
 require_once __DIR__ . '/vendor/autoload.php';


function acf_admin_notice() {
	?>
	<div class="updated error is-dismissible">
         <p>
			<?php echo __( 'Bitte installiere / aktiviere ACF, damit "POST TO IMAGE" funktioniert.', 'title_to_postimage' ); ?>
		 </p>
    </div>
	<?php
}

if( !empty( ACF_SUPPORT ) ) {
	require_once 'acf/acf.php';

	require_once __DIR__ . '/includes/save_post.php';

	require_once __DIR__ . '/classes/admin/class-canvas.php';
	require_once __DIR__ . '/classes/admin/class-initital.php';
	require_once __DIR__ . '/classes/admin/class-menu.php';
	require_once __DIR__ . '/classes/admin/class-scripts.php'; 

	add_action('init', function() {
		$adminInitial = new \LWM\PTI\Initial();
		$adminInitial->init();

		$adminMenu = new \LWM\PTI\Menu();
		$adminMenu->init();

		$adminScripts = new \LWM\PTI\Scripts();
		$adminScripts->init();
	});
} else {
	add_action( 'admin_notices', 'acf_admin_notice' );
}

require_once __DIR__ . '/classes/admin/class-states.php';

function activate_ttpi() {
	\LWM\PTI\States::activate();
}

function deactivate_ttpi() {
	\LWM\PTI\States::deactivate();
}

register_activation_hook( __FILE__, 'activate_ttpi' );
register_deactivation_hook( __FILE__, 'deactivate_ttpi' );

/**
 * plugin updater
 * 
 */
PucFactory::buildUpdateChecker(
	'http://plugins.lightweb-media.de/api?action=get_metadata&slug=title_to_postimage_plugin', //Metadata URL.
	__FILE__, //Full path to the main plugin file.
	'title_to_postimage_plugin' //Plugin slug. Usually it's the same as the name of the directory.
);